/*------------------------------------------------------*
 | Author: Maurizio Loreti, aka MLO or (HAM) I3NOO      |
 | Work:   University of Padova - Department of Physics |
 |         Via F. Marzolo, 8 - 35131 PADOVA - Italy     |
 | Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102 |
 | EMail:  loreti@pd.infn.it                            |
 | WWW:    http://www.pd.infn.it/~loreti/mlo.html       |
 *------------------------------------------------------*

 Header file for general AVL balanced binary tree routines,
 coded in "avl.c".

 $Id: avl.h,v 1.1 2001/01/17 11:20:10 loreti Exp $

 *------------------------------------------------------*/

#ifndef AVL_H_
#define AVL_H_

/**
 | The user MUST define in "item.h" a typedef <something> item; where
 | "item" is the user data to be included in the binary tree
**/

#include "item.h"

/**
 | Struct sNode: a node in the binary tree
**/

typedef struct sNode {
  struct sNode *left;
  struct sNode *right;
  int           balance;
  item         *pItem;
} node;

/**
 | Symbolic constants for tree traversal order
**/

typedef enum eOrder {
  INORDER, PREORDER, POSTORDER
} order;

/**
 | Shortcuts
**/

#define OK    0
#define ERROR 1

/**
 | Prototypes
**/

item *findAvl(node *, const item *, int(const item *, const item *));
int   insertAvl(node **, item *, node **, int(const item *, const item *));
void  removeAvl(node *, void(item *));
int   removeFromAvl(node **, const item *, int(const item *, const item *),
                   void(item *));
void  traverseAvl(const node *, const order, void(const item *));

#endif /* AVL_H_ */
