//     Author: Maurizio Loreti, aka MLO or (HAM) I3NOO
//     Work:   University of Padova - Department of Physics
//             Via F. Marzolo, 8 - 35131 PADOVA - Italy
//     Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102
//     EMail:  loreti@pd.infn.it
//     WWW:    http://www.pd.infn.it/~loreti/mlo.html
//
// -------------------------------------------------------------------
//
// A simple word counting procedure that uses the C++ Standard Library
//
// $Id: stlMain.cxx,v 1.3 2005/01/21 08:12:19 loreti Exp $
//
// -------------------------------------------------------------------

#include <iostream>
#include <fstream>
#include <string>
#include <map>
#include <cctype>

using std::cerr;
using std::cout;
using std::ifstream;

typedef std::map< std::string, unsigned int > msui;

const char *getWord(
  ifstream &ifs
) {
  // Reads from the input stream a "word", defined as an alphabetic
  // character followed by other alphabetic characters and/or numbers
  // and/or underscores); and returns a pointer to the word itself,
  // converted to lowercase.

  static const size_t bufferLength(128);
  static char         buffer[bufferLength];
  static size_t       next(0);
  static bool         eof_reached(false);

  if (eof_reached) return 0;

  for (;;) {
    ifstream::int_type c;

    if ((c = ifs.get()) == ifstream::traits_type::eof()) {
      if (next) {
        eof_reached  = true;
        buffer[next] = '\0';
        return buffer;
      } else {
        return 0;
      }
    }

    c = tolower(static_cast<unsigned char>(c));
    if (!next) {
      if (isalpha(static_cast<unsigned char>(c))) buffer[next++] = c;
    } else {
      if (isalpha(static_cast<unsigned char>(c))   ||
          isdigit(static_cast<unsigned char>(c))   ||   c == '_') {
        buffer[next++] = c;
      } else {
        next = buffer[next] = '\0';
        return buffer;
      }
    }
  }
}

int main(
  int   argc,
  char *argv[]
) {
  if (argc < 2) {
    cerr << "Usage: " << argv[0] << " <1 or more file names>\n";
  } else {
    unsigned long nWords = 0;
    msui dict;

    while (--argc) {
      ifstream ifs(*++argv);

      if (ifs) {
        cout << "Working on " << *argv << " ...";
        cout.flush();

        while (const char *pc = getWord(ifs)) {
          ++nWords;
          ++(dict[pc]);
        }
        ifs.close();
        cout << " done." << std::endl;

      } else {
        cerr << "Error opening file \"" << *argv << "\"\n";
      }
    }

    cerr << "\n  Found " << nWords <<" words ("
         << dict.size() << " different)\n\n";

    if (dict.size()) {
      msui::const_iterator it, finis = dict.end();

      cout << "* In alphabetical order:\n";
      for (it = dict.begin();   it != finis;   ++it) {
        cout << "- " << it->first << ": " << it->second
             << " times\n";
      }
    }
  }
}
